const vscode = require('vscode');
const path = require('path');
const fs = require('fs');
const request = require('./request-2.88.1/index.js');

function getExtensionFileAbsolutePath(context, relativePath) {
    return path.join(context.extensionPath, relativePath);
}
// /**
//  * 从某个HTML文件读取能被Webview加载的HTML内容
//  * @param {*} context 上下文
//  * @param {*} templatePath 相对于插件根目录的html文件相对路径
//  */
function getWebViewContent(context, templatePath) {
    const resourcePath = getExtensionFileAbsolutePath(context, templatePath);
    const dirPath = path.dirname(resourcePath);
    let html = fs.readFileSync(resourcePath, 'utf-8');
    // vscode不支持直接加载本地资源，需要替换成其专有路径格式，这里只是简单的将样式和JS的路径替换
    html = html.replace(/(<link.+?href="|<script.+?src="|<img.+?src=")(.+?)"/g, (m, $1, $2) => {
        return $1 + vscode.Uri.file(path.resolve(dirPath, $2)).with({ scheme: 'vscode-resource' }).toString() + '"';
    });
    return html;
}

function getProjectPath(document) {
    if (!document) {
        document = vscode.window.activeTextEditor ? vscode.window.activeTextEditor.document : null;
    }
    if (!document) {
        vscode.window.showInformationMessage('当前激活的编辑器不是文件或者没有文件被打开！');
        return '';
    }
    const currentFile = (document.uri ? document.uri : document).path;
    let projectPath = null;
    let workspaceFolders = vscode.workspace.workspaceFolders.map((item) => item.uri.path);
    // 由于存在Multi-root工作区，暂时没有特别好的判断方法，先这样粗暴判断
    // 如果发现只有一个根文件夹，读取其子文件夹作为 workspaceFolders
    if (workspaceFolders.length == 1 && workspaceFolders[0] === vscode.workspace.rootPath) {
        const rootPath = workspaceFolders[0];
        var files = fs.readdirSync(rootPath);
        workspaceFolders = files.filter((name) => !/^\./g.test(name)).map((name) => path.resolve(rootPath, name));
        // vscode.workspace.rootPath会不准确，且已过时
        // return vscode.workspace.rootPath + '/' + this._getProjectName(vscode, document);
    }
    console.log(workspaceFolders);
    workspaceFolders.forEach((folder) => {
        if (currentFile.indexOf(folder) === 0) {
            projectPath = folder.substr(1);
            console.log(projectPath);
        }
    });
    if (!projectPath) {
        vscode.window.showInformationMessage('获取工程根路径异常！');
        return '';
    }
    return projectPath;
}

function invokeCallback(panel, message, resp) {
    panel.webview.postMessage({ cmd: 'vscodeCallback', cbid: message.cbid, data: resp });
}

const messageHandler = {
    /**
     * 上传文件
     * @param {*} global
     * @param {*} message
     */
    uploadFile(global, message) {
        if(message.address.substr(0,1)==='/'){
            message.address = message.address.substring(1)
        }
        const statRes = fs.statSync(message.address);
        if (!statRes.isFile()) {
            vscode.window.showInformationMessage('rpk 包选择不正确');
            return;
        }
        request.post(
            `${message.domain}/rpkDebug/uploadAndPush`,
            {
                formData: {
                    file: fs.createReadStream(message.address),
                    bindNumberList: message.bindnumber,
                    clearData: message.clearData
                },
                json: true
            },
            function (err, res, body) {
                console.log(`返回: ${JSON.stringify(body)}`);
                invokeCallback(global.panel, message, {
                    body: body
                });
            }
        );
    },
    /**
     * 卸载手表应用
     * @param {*} global 
     * @param {*} message 
     */
    uninstall(global, message) {
        request.post(
            `${message.domain}/rpkDebug/del`,
            {
                body: {
                    bindNumberList: message.bindnumber,
                    packageName: message.packageName
                },
                headers:{
                    'Content-Type': 'application/json'
                },
                json: true
            },
            function (err, res, body) {
                console.log(`返回: ${JSON.stringify(body)}`);
                invokeCallback(global.panel, message, {
                    body: body
                });
            }
        );
    },

    /**
     * 获取信息
     * @param {*} global
     * @param {*} message
     */
    getRpkPath(global, message) {
        const lists = fs.readdirSync(`${global.projectPath}/dist`);
        let rpks = lists.filter((r) => r.endsWith('.rpk'));
        if (rpks.length === 0) {
            return;
        }
        rpks = rpks.find((r) => r.includes('release'));
        if(rpks){
            const path = `${global.projectPath}/dist/${rpks}`;
            const stats = fs.statSync(path);
            invokeCallback(global.panel, message, {
                projectPath: global.projectPath,
                size: stats.size,
                rpk: path
            });
        }
    },
    /**
     * 生成配置文件
     * @param {*} global
     * @param {*} message
     */
    createConfig(global, message) {
        const configPath = `${global.projectPath}/ide-config.json`
        fs.writeFileSync(configPath, JSON.stringify({
            "bindnumber":message.bindnumber,
            "clearData":message.clearData,
            "env":message.env
        }))
    },
    /**
     * 读取配置文件
     * @param {*} global
     * @param {*} message
     */
    getConfig(global, message) {
        const configPath = `${global.projectPath}/ide-config.json`
        const config = fs.existsSync(configPath)?fs.readFileSync(configPath,'utf-8'):null;
        invokeCallback(global.panel, message, {
            data: config
        });
    },
    /**
     * 手动选择文件
     * @param {*} global
     * @param {*} message
     */
    async selectRpk(global, message) {
        const uris = await vscode.window.showOpenDialog({
            canSelectFolders: false, // 是否可以选择文件夹
            canSelectMany: false, // 是否可以选择多个文件
            defaultUri: vscode.Uri.file(`${global.projectPath}/dist/`),
            openLabel: '选择文件',
            filters: {
                rpk: ['rpk'] // 文件类型过滤
            }
        });
        if (!uris || !uris.length) {
            return;
        }
        invokeCallback(global.panel, message, {
            projectPath: global.projectPath,
            uris
        });
    }
};

/**
 * 插件被激活时触发，所有代码总入口
 * @param {*} context 插件上下文
 */
exports.activate = function (context) {
    console.log('恭喜，您的扩展“vscode-plugin-demo”已被激活！');
    // 注册命令
    context.subscriptions.push(
        vscode.commands.registerCommand('extension.sendToWatch', function (uri) {
            const projectPath = getProjectPath(uri);
            if (!projectPath) return;
            const panel = vscode.window.createWebviewPanel(
                'testWebview', // viewType
                '上传rpk', // 视图标题
                vscode.ViewColumn.One, // 显示在编辑器的哪个部位
                {
                    enableScripts: true, // 启用JS，默认禁用
                    retainContextWhenHidden: true // webview被隐藏时保持状态，避免被重置
                }
            );
            let global = { projectPath, panel };
            panel.webview.html = getWebViewContent(context, 'src/view/index.html');
            panel.webview.onDidReceiveMessage(
                (message) => {
                    if (messageHandler[message.cmd]) {
                        messageHandler[message.cmd](global, message);
                    } else {
                        // util.showError(`未找到名为 ${message.cmd} 回调方法!`);
                    }
                },
                undefined,
                context.subscriptions
            );
        })
    );
};

exports.deactivate = function () {
    console.log('您的扩展“vscode-plugin-demo”已被释放！');
};
